#!/usr/bin/haserl --upload-limit=2048 --upload-dir=/tmp
<%
# in this case content type needs to be /text/html and not application/json,
# otherwise ext will enclose the response in <pre> tags
echo -en "Content-Type: text/html\r\n\r\n"

. ./errlib.sh
. ./jsonlib.sh
. ./util.sh
. ./setdate.sh >/dev/null 2>&1
. ./sshconfig.sh >/dev/null 2>&1
. ./namelib.sh

LOCKFILE="/var/lock/systemupdate"

ADDON_PROPDIR="usr/share/dss/data/savedprops/"

move_to() {
    if [[ -z "$1" ]] || [[ -z "$2" ]]; then
        return $ERR_USAGE
    fi

    OUT=$(mv "$1" "$2" 2>&1)
    if [[ $? -ne 0 ]]; then
        log "Could not restore $2: $OUT"
        return $ERR_SOFTWARE
    fi

    log "Restored $2"

    return $ERR_OK
}


log "Received request to restore system from backup"

if [[ -n "$CONTENT_LENGTH" ]]; then
    if [[ $CONTENT_LENGTH -gt 2097152 ]]; then
        json_error $ERR_INVALID "Maximum upload size (2MB) exceeded"
        log "Maximum upload size exceeded ($CONTENT_LENGTH > 2MB)"
        exit 1
    fi
fi

if [[ -z "$HASERL_upload_path" ]]; then
    json_error $ERR_FILENOTFOUND "No file uploaded"
    log "Restore requested but no file uploaded"
    exit 1
fi

tar -tf "$HASERL_upload_path" > /dev/null 2>&1
if [[ $? -ne 0 ]]; then
    json_error $ERR_INVALID "Uploaded file is not a gzipped tar archive"
    log "Backup file is invalid, not a gzipped tar archive"
    exit 1
fi

if [[ -e $LOCKFILE ]]; then
    json_error $ERR_DOWAIT "Upate or backup is in progress, please try again \
                            later"
    log "System update is in progress, restore not allowed"
    exit 1
fi

touch $LOCKFILE

LIST=$(tar -tf "$HASERL_upload_path")
NET_REDIRECT=""
REDIRECT_CODE=$ERR_OK
ERROR=0
EXTRACT_DIR="$HASERL_upload_path.extract"
mkdir -p "$EXTRACT_DIR" > /dev/null 2>&1
if [[ $? -ne 0 ]]; then
    json_error $ERR_IOERR "Could not create directory for archive data"
    log "Could not create directory for unpacking archive data"
    rm -f $LOCKFILE
    exit 1
fi

control_dss $RUNIT_STATUS
DSS_STATUS=$?

control_dss $RUNIT_DOWN

for f in $LIST
do
    log "Processing $f"
    if [[ "$f" == "usr/share/dss/data/apartment.xml" ]]; then
        log "Found apartment.xml in backup"
        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "usr/share/dss/data/apartment.xml" \
            -O  > "$EXTRACT_DIR/apartment.xml"
        if [[ $? -ne 0 ]]; then
            log "Failed to extract $f"
            ERROR=1
            continue
        fi

        check_xml_file "$EXTRACT_DIR/apartment.xml"
        if [[ $? -eq $ERR_OK ]]; then
            move_to "$EXTRACT_DIR/apartment.xml" \
                    "/usr/share/dss/data/apartment.xml"
            if [[ $? -ne $ERR_OK ]]; then
                ERROR=1
            fi
        else
            log "Will not restore $f because XML is not well formed"
            ERROR=1
        fi
    elif [[ "$f" == "usr/share/dss/data/config.d/location.xml" ]]; then
        log "Found location.xml in backup"
        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "usr/share/dss/data/config.d/location.xml" \
            -O  > "$EXTRACT_DIR/location.xml"
        if [[ $? -ne 0 ]]; then
            log "Failed to extract $f"
            ERROR=1
            continue
        fi

        check_xml_file "$EXTRACT_DIR/location.xml"
        if [[ $? -eq $ERR_OK ]]; then
            move_to "$EXTRACT_DIR/location.xml" \
                    "/usr/share/dss/data/config.d/location.xml"
            if [[ $? -ne $ERR_OK ]]; then
                ERROR=1
            fi
        else
            log "Will not restore $f because XML is not well formed"
            ERROR=1
        fi
    elif [[ "$f" == "etc/hostname" ]]; then
        log "Found hostname in backup"
        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "etc/hostname" -O  > "$EXTRACT_DIR/hostname"
        if [[ $? -ne 0 ]]; then
            log "Failed to extract $f"
            ERROR=1
            continue
        fi

        check_host_name "$EXTRACT_DIR/hostname"
        if [[ $? -eq $ERR_OK ]]; then
            HOST_NAME=$(cat "$EXTRACT_DIR/hostname")
            NET_REDIRECT=$(set_hostname "$HOST_NAME" "keepbackupiface")
            REDIRECT_CODE=$?
            if [[ $REDIRECT_CODE -ne $ERR_OK ]] && \
               [[ $REDIRECT_CODE -ne $ERR_REDIRECT ]] && \
               [[ $REDIRECT_CODE -ne $ERR_REDIRECT_NAME ]]; then
                log "Failed to set hostname $HOST_NAME: code $REDIRECT_CODE"
                ERROR=1
                NET_REDIRECT=""
            fi

            set_upnp_friendly_name "$HOST_NAME" > /dev/null 2>&1
            if [[ $? -ne $ERR_OK ]]; then
                log "Failed to set UPnP friendly name $HOST_NAME"
                ERROR=1
            fi
        else
            log "Will not restore $f because it contains an invalid \
                 configuration"
        fi
    elif [[ "$f" == "etc/default/dropbear" ]]; then
        log "Found dropbear ssh configuration in backup"
        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "etc/default/dropbear" -O  > "$EXTRACT_DIR/dropbear"
        if [[ $? -ne 0 ]]; then
            log "Failed to extract $f"
            ERROR=1
            continue
        fi

        check_ssh_config "$EXTRACT_DIR/dropbear"
        if [[ $? -eq $ERR_OK ]]; then
            RESTORE_SSH=$(grep NO_START "$EXTRACT_DIR/dropbear" \
                          | cut -d '=' -f 2)
            set_ssh $RESTORE_SSH
            if [[ $? -ne $ERR_OK ]]; then
                ERROR=1
                log "Failed to restore SSH settings"
                continue
            else
                log "Restored $f"
            fi
        else
            ERROR=1
            log "Will not restore $f because it contains an invalid \
                 configuration"
        fi
    elif [[ "$f" == "etc/timezone" ]]; then
        log "Found time zone settings in backup"
        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "etc/timezone" -O  > "$EXTRACT_DIR/timezone"
        if [[ $? -ne 0 ]]; then
            log "Failed to extract $f"
            ERROR=1
            continue
        fi
        RESTORE_TZ=$(cat "$EXTRACT_DIR/timezone")
        set_timezone "$RESTORE_TZ"
        if [[ $? -ne $ERR_OK ]]; then
            ERROR=1
            if [[ $RETCODE -eq $ERR_NOZONES ]]; then
                log "Requested time zone $RESTORE_TZ is not installed"
            elif [[ $RETCODE -eq $ERR_LINK ]]; then
                log "Could not set requested zone $RESTORE_TZ, link failed"
            fi
        else
            log "Restored $f"
        fi
    elif [[ "$f" == "etc/default/ntpclient" ]]; then
        log "Found ntp settings in backup"
        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "etc/default/ntpclient" -O  > "$EXTRACT_DIR/ntpclient"
        if [[ $? -ne 0 ]]; then
            log "Failed to extract $f"
            ERROR=1
            continue
        fi

        check_ntp_config "$EXTRACT_DIR/ntpclient"
        if [[ $? -ne $ERR_OK ]]; then
            log "Will not restore $f because it contains an invalid \
                 configuration"
            ERROR=1
            continue
        else
            log "Restored $f"
        fi

        NTP_EN=$(egrep ENABLED "$EXTRACT_DIR/ntpclient" | \
                 cut -d '=' -f 2 | egrep '(yes|no)')

        set_ntp "$NTP_EN"
        if [[ $? -ne $ERR_OK ]]; then
            log "Failed to apply NTP settings from backup"
            ERROR=1
        fi
    elif [[ "$f" == "etc/msmtprc" ]]; then
        log "Found e-mail settings in backup"
        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "etc/msmtprc" -O  > "$EXTRACT_DIR/msmtprc"
        if [[ $? -ne 0 ]]; then
            log "Failed to extract $f"
            ERROR=1
            continue
        fi
        move_to "$EXTRACT_DIR/msmtprc" "/etc/msmtprc"
    elif [[ "$f" == "etc/default/mail" ]]; then
        log "Found e-mail settings in backup"
        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "etc/default/mail" -O  > "$EXTRACT_DIR/mail"
        if [[ $? -ne 0 ]]; then
            log "Failed to extract $f"
            ERROR=1
            continue
        fi
        move_to "$EXTRACT_DIR/mail" "/etc/default/mail"
    elif [[ "$f" == "etc/dsa/dsa.cfg" ]]; then
        log "Found dSA settings in backup, extracting apartment id..."
        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "etc/dsa/dsa.cfg" -O  > "$EXTRACT_DIR/dsa.cfg"
        if [[ $? -ne 0 ]]; then
            log "Failed to extract $f"
            ERROR=1
            continue
        fi
        APT_ID=`sed -n 's/apartmentid = \([0-9]\+\);$/\1/;T;p' $EXTRACT_DIR/dsa.cfg` 2>/dev/null
        if [[ -n "$APT_ID" ]]; then
            if [[ -f "/etc/dsa/dsa.cfg" ]]; then
                sed -i "s/\(apartmentid = \)\([0-9]\+\);$/\1$APT_ID;/" /etc/dsa/dsa.cfg 
                log "Restored apartment id $APT_ID"
            else
                move_to "$EXTRACT_DIR/dsa.cfg" "/etc/dsa/dsa.cfg"
                log "/etc/dsa/dsa.cfg was missing, restored whole configuration"
            fi
            sv restart dsa >/dev/null 2>&1
        else
            log "Could not extract apartment id from backup"
        fi
    elif [[ "$(echo "$f" | head -c ${#ADDON_PROPDIR})" == \
            "$ADDON_PROPDIR" ]]; then
        FILENAME=$(basename "$f") 2>/dev/null
        if [[ -z "$FILENAME" ]]; then
            log "Failed to extract base name from $f"
            ERROR=1
            continue
        fi

        tar -xf "$HASERL_upload_path" -C "$EXTRACT_DIR" \
            "$f" -O  > "$EXTRACT_DIR/$FILENAME"

        check_xml_file "$EXTRACT_DIR/$FILENAME"
        if [[ $? -eq $ERR_OK ]]; then
            move_to "$EXTRACT_DIR/$FILENAME" \
                    "/$ADDON_PROPDIR$FILENAME"
            if [[ $? -ne $ERR_OK ]]; then
                ERROR=1
            fi
        else
            log "Will not restore $f because XML is not well formed"
            ERROR=1
        fi
    fi
done

sync
control_dss $DSS_STATUS
# give dSS time to start, otherwise UI will reload too fast and report an error
sleep 6

if [[ $ERROR -eq 0 ]]; then
    if [[ -n "$NET_REDIRECT" ]]; then
        json_redirect $REDIRECT_CODE "`echo $NET_REDIRECT | cut -d ':' -f 1`" \
                      "`echo $NET_REDIRECT | cut -d ':' -f 2`" \
                      "System successfully restored, redirecting to new IP"
        log "System restored, redirect needed $NET_REDIRECT"
        down_backup_if
    else
        json_ok
        log "System restored"
    fi
else
    if [[ -n "$NET_REDIRECT" ]]; then
        json_redirect $REDIRECT_CODE "`echo $NET_REDIRECT | cut -d ':' -f 1`" \
                      "`echo $NET_REDIRECT | cut -d ':' -f 2`" \
                      "One or more files could not be restored, \
                       redirecting to new IP"
        log "System restore had errors, please review the log. Redirect \
             needed $NET_REDIRECT"
        sleep 1
        down_backup_if
    else
        json_error $ERR_SOFTWARE "One or more files could not be restored"
        log "System restore had errors, please review the log"
    fi
fi


rm -rf "$EXTRACT_DIR"
rm -f $LOCKFILE

%>

